/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2005  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/


#include <boost/shared_ptr.hpp>
#include "eventmanager.h"
#include "gameevent.h"

#include <algorithm>
#include <functional>
#include <list>

#include "melee.h"
#include "melee/mframe.h"
#include "games/ggob.h"
#include "melee/mship.h"

#include "python/python_class.h"
//////////////////////////////////////////////////////////////////
std::list<TGameEventPtr > EventManager::g_events;
std::set<SpaceLocation*> EventManager::m_allSpaceLocations;

//////////////////////////////////////////////////////////////////

void EventManager::clean()
{
  STACKTRACE;
  for(twl::EventList::iterator i = g_events.begin(); i!= g_events.end();i++)
    {
      (*i)->die();
    }
  g_events.clear();
}

int EventManager::setSingleTimeEvent(double time, const std::string& script)
{
  STACKTRACE;
  TEventSingleTime* e = new TEventSingleTime();
  boost::shared_ptr<TGameEvent> ev( e );
  int id = e->setUp((int)time,script);
  g_events.push_back(ev);
  return id;
}

int EventManager::setRepeatableTimeEvent(double time, const std::string& script)
{
  STACKTRACE;
  TEventRepeatableTime* e = new TEventRepeatableTime();
  boost::shared_ptr<TGameEvent> ev( e );
  int id = e->setUp((int)time,script);
  g_events.push_back(ev);
  return id;
}

struct find_event_by_id : public std::binary_function< boost::shared_ptr<TGameEvent>, int, bool>
{
  bool operator() ( TGameEventPtr e, int id) const
  {
    if(e->exists() && e->getID()==id)
      {
	return true;
      }
    return false;
  }
};

void EventManager::disableEvent(double id)
{
  STACKTRACE;
  std::list<boost::shared_ptr<TGameEvent> >::iterator e = std::find_if(g_events.begin(), 
								       g_events.end(), 
								       std::bind2nd(find_event_by_id(),id));
  if(e!=g_events.end()&&(*e)->exists())
    {
      (*e)->die();
    }
}


int EventManager::shipDeathById(double id, const std::string& script )
{
  STACKTRACE;
  int ret = -1;
  SpaceLocation* l  =  twl::get_location((int)id);
  if(l)
    {
      TEventShipDeathById* e = new TEventShipDeathById();
      ret = e->setUp(script);
      boost::shared_ptr<TGameEvent> ev (e);
      (l)->add_event(ev);
    }
  return ret;
}

int EventManager::shipDeathByType(const std::string& type, const std::string& script )
{
  STACKTRACE;
  TEventShipDeathByType* e = new TEventShipDeathByType();
  boost::shared_ptr<TGameEvent> ev( e );
  int ret = e->setUp(type, script);
  g_events.push_back(ev);
  return ret;
}


template <class T>
int EventManager::distanceXThen(double obj, double x, double y, double dist, const std::string& script)
{
  STACKTRACE;
  std::set<SpaceLocation*>::iterator i = twl::get_location< std::set<SpaceLocation*>::iterator > (m_allSpaceLocations.begin(),m_allSpaceLocations.end(), (int)obj);
  if(i!=m_allSpaceLocations.end())
    {
      T* e = new T();
      boost::shared_ptr<TGameEvent> ev( e );
      int ret = e->setUp(*i, Vector2(x,y), dist, script);
      g_events.push_back(ev);
      return ret;
    }
  return -1;
}

int EventManager::distanceMoreThen(double obj, double x, double y, double dist, const std::string& script)
{
  STACKTRACE;
  return distanceXThen<TEventDistanceMoreThen1Obj>(obj, x, y, dist, script);
}

int EventManager::distanceLessThen(double obj, double x, double y, double dist, const std::string& script)
{
  STACKTRACE;
  return distanceXThen<TEventDistanceLessThen1Obj>(obj, x, y, dist, script);
}


template <class T>
int EventManager::distanceXThen(double obj1, double obj2, double dist, const std::string& script)
{
  STACKTRACE;
  std::set<SpaceLocation*>::iterator i1 = twl::get_location< std::set<SpaceLocation*>::iterator > (m_allSpaceLocations.begin(),m_allSpaceLocations.end(), (int)obj1);
  std::set<SpaceLocation*>::iterator i2 = twl::get_location< std::set<SpaceLocation*>::iterator > (m_allSpaceLocations.begin(),m_allSpaceLocations.end(), (int)obj2);
  if(i1!=m_allSpaceLocations.end()&&i2!=m_allSpaceLocations.end())
    {
      T* e = new T();
      boost::shared_ptr<TGameEvent> ev( e );
      int ret = e->setUp(*i1,*i2, dist, script);
      g_events.push_back(ev);
      return ret;
    }
  return -1;
}

int EventManager::distanceMoreThen(double obj1, double obj2, double dist, const std::string& script)
{
  STACKTRACE;
  return distanceXThen<TEventDistanceMoreThen2Obj>(obj1, obj2, dist, script);
}

int EventManager::distanceLessThen(double obj1, double obj2, double dist, const std::string& script)
{
  STACKTRACE;
  return distanceXThen<TEventDistanceLessThen2Obj>(obj1, obj2, dist, script);
}

int EventManager::jumpToSystem(const std::string& system, const std::string& script)
{
	STACKTRACE;
	TEventJumpToSystem* e = new TEventJumpToSystem();
    boost::shared_ptr<TGameEvent> ev( e );
    int ret = e->setUp(system, script);
    g_events.push_back(ev);
    return ret;
}

void EventManager::AddEventCondition(double id, const std::string& script)
{
  std::list<boost::shared_ptr<TGameEvent> >::iterator e = std::find_if(g_events.begin(), 
								       g_events.end(), 
								       std::bind2nd(find_event_by_id(),id));
  if(e!=g_events.end()&&(*e)->exists())
    {
      (*e)->add_condition(script);
    }  
}
//////////////////////////////////////////////////////////////////
//  Calculate events                                            //
//////////////////////////////////////////////////////////////////

static void calcevent(boost::shared_ptr<TGameEvent> e)
{
  if(e->exists())
    {
      e->calculate();
    }
}

namespace twl
{
  bool isexists::operator() (boost::shared_ptr<TGameEvent> e) const
  {
    if(e->exists())
      {
	return true;
      }
    return false;
  }
  EventList::iterator has_event( EventList::iterator begin,
				 EventList::iterator end,
				 int type)
  {
    for (EventList::iterator i = begin; i!= end;i++)
      {
	if( (*i)->type() == type && (*i)->exists())
	  return i;
      }
    return end;
  }

  template<class T>
  T get_location(T begin,  T end,  int id)
  {
    for(T i = begin; i!= end;i++)
      {
	if( (*i)->get_id()==id && (*i)->exists())
	  {
	    return i;
	  }
      }
    return end;
  }

  SpaceLocation* get_location(int id)
  {
    std::set<SpaceLocation*>::iterator i = twl::get_location< std::set<SpaceLocation*>::iterator > (EventManager::m_allSpaceLocations.begin(),EventManager::m_allSpaceLocations.end(), id);
    if(i!= EventManager::m_allSpaceLocations.end())
      {
	return *i;
      }
    return NULL;
  }
};

void EventManager::calculate()
{
  STACKTRACE;
  std::for_each(g_events.begin(), g_events.end(), calcevent);  
  g_events.erase(std::remove_if(g_events.begin(), g_events.end(), std::not1(twl::isexists())),g_events.end());
}
