/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2005  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/


#ifndef __TW_LIGHT_EVENTMANAGER_H__
#define __TW_LIGHT_EVENTMANAGER_H__

#include <boost/shared_ptr.hpp>
#include <string>
#include <list>
#include <set>

class SpaceLocation;
typedef boost::shared_ptr<SpaceLocation> SpaceLocationPtr;

class TGameEvent;
typedef boost::shared_ptr<TGameEvent> TGameEventPtr;

/// event API for python
class EventManager
{
 public:
  /// Destroy all events
  static void clean();
  
  /// Create one time time event
  ///
  /// @param time - time when call code
  /// @script     - function to call
  /// @return     - event id
  static int setSingleTimeEvent(double time, const std::string& script);
  
  /// Create one time time event
  ///
  /// @param time - time between calls
  /// @script     - function to call
  /// @return     - event id
  static int setRepeatableTimeEvent(double time, const std::string& script);
  
  /// Remove event
  /// 
  /// @param id - id of event
  static void disableEvent(double id);


  /// Call function script when ship named 'name' die
  ///
  /// @param id     - ship ID
  /// @param script - function to call
  static int shipDeathById(double, const std::string& script );
  
  /// Call function script when ship whose type type die
  ///
  /// @param type   - ship build type 
  /// @param script - function to call
  static int shipDeathByType(const std::string& type, const std::string& script );

  /// Call function if distance between object and x,y more then dist
  ///
  /// If obj dies, this event should die as well
  ///
  /// @param obj    - space location id (number you recieve by create* functions)
  /// @param x      - x coord of point
  /// @param y      - y coord of point
  /// @param script - function to call
  /// @return event id
  static int distanceMoreThen(double obj, double x, double y, double dist, const std::string& script);

  /// Call function if distance between object and x,y less then dist
  ///
  /// If obj dies, this event should die as well
  ///
  /// @param obj    - space location id (number you recieve by create* functions)
  /// @param x      - x coord of point
  /// @param y      - y coord of point
  /// @param script - function to call
  /// @return event id
  static int distanceLessThen(double obj, double x, double y, double dist, const std::string& script);

  /// Call function if distance between object1 and object2 more then dist
  ///
  /// If obj dies, this event should die as well
  ///
  /// @param obj    - space location id (number you recieve by create* functions)
  /// @param x      - x coord of point
  /// @param y      - y coord of point
  /// @param script - function to call
  /// @return event id
  static int distanceMoreThen(double obj1, double obj2, double dist, const std::string& script);

  /// Call function if distance between object and object2 less then dist
  ///
  /// If obj dies, this event should die as well
  ///
  /// @param obj    - space location id (number you recieve by create* functions)
  /// @param x      - x coord of point
  /// @param y      - y coord of point
  /// @param script - function to call
  /// @return event id
  static int distanceLessThen(double obj1, double obj2, double dist, const std::string& script);

  /// Call function if player enter star system
  ///
  ///
  /// @param system - system where player must enter
  /// @param script - function to call
  /// @return event id
  static int jumpToSystem(const std::string& system, const std::string& script);

  /// Additional condition must be true to trigger event
  static void AddEventCondition(double id, const std::string& script);

#ifndef SWIG
  template <class T>
  static int distanceXThen(double obj, double x, double y, double dist, const std::string& script);

  template <class T>
  static int distanceXThen(double obj1, double obj2, double dist, const std::string& script);

  static std::list<TGameEventPtr > g_events;
  static std::set<SpaceLocation*> m_allSpaceLocations;
  /// Should be called every frame for calculating events
  static void calculate();
#endif

};

#ifndef SWIG

namespace twl
{

  typedef std::list< TGameEventPtr > EventList;

  struct isexists : public std::unary_function< boost::shared_ptr<TGameEvent>, bool>
  {
    bool operator() (TGameEventPtr e) const;
  };

/// Check if event with type type present
///
/// @param begin - begin of sequence
/// @param end   - end of sequence
/// @return      - finded item or end if not found
  EventList::iterator has_event( EventList::iterator begin,
				 EventList::iterator end,
				 int type);

  /// Get SpaceLocation by it's id
  ///
  /// @param begin 
  /// @param end
  /// @param id id of space object, returned by create* function
  template<class T>
    T get_location(T begin, T end, int id);
  
  SpaceLocation* get_location(int id);
};

#endif

#endif

