/*
This file is part of "TW-Light" 
                    http://tw-light.berlios.de/
Copyright (C) 2001-2005  TimeWarp development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/


#ifndef __TW_LIGHT_GAMEACTION_H__
#define __TW_LIGHT_GAMEACTION_H__

#include <string>
#include <vector>

/// Actions that can be made from python scripts
class GameAction
{
 public:
  /// Clear all variables
  static void clean();

  /// Add new star system to the game
  ///
  /// @param name - star system name, should be unique serve as id
  /// @param x    - x coordinate of star system in quadrant map
  /// @param y    - y coordinate of star system in quadrant map
  static void add_system(const std::string& name, double x, double y);
  
  /// Add new planet
  ///
  /// @param system  - star system name to add planet
  /// @param name    - planet name (should be unique in star system range)
  /// @param x planet coordinate in system
  /// @param y planet coordinate in system
  /// @return planet id
  static int add_planet(const std::string& system, const std::string& name, double x, double y);
  
  /// Add station that move on planet orbit
  ///
  /// note: this ugly function  will be changed
  ///
  /// @param system             - star system name
  /// @param planet             - planet name
  /// @param datafile           - path to datafile
  /// @param station_sprite     - station sprite name in datafile
  /// @param background_picture - background pickture when player at the station
  /// @buildtype                - what kind of ships player can build here
  /// @commander_dialog         - commander dialog file
  /// @return station id
  static int add_orbiter_station(const std::string& system, 
			     const std::string& planet, 
			     const std::string& station_sprite,
			     const std::string& background_picture,
			     const std::string& buildtype,
			     const std::string& commander_dialog);

  /// Add asteroid to the star system
  ///
  /// @param system star system where to add
  /// @param x The asteroid will be created in (x,y) position and will be also be respawned here -1 for random
  /// @param y The asteroid will be created in (x,y) position and will be also be respawned here -1 for random
  /// @param oangle - angle in rad where steroid pointed -1 for random
  /// @param speed asteroid speed -1 for random
  /// @return asteroid id
  static int add_asteroid(const std::string& system, double x = -1, double y = -1, double oangle = -1, double speed = -1);
  
  /// Add player to the game, should be called only once per game
  ///
  /// @param system where player appear
  /// @param x player coordinate in system
  /// @param y player coordinate in system
  /// @return player id
  static int add_player(const std::string& system, const std::string&, double x, double y);

  /// Add computer ship to the game
  ///
  /// @param system where player appear
  /// @param x player coordinate in system
  /// @param y player coordinate in system
  /// @param team team code
  /// @return ship id
  static int add_ship(const std::string& system, const std::string& shptype, double x, double y, double team);

  //////////////////////////////////////////////////////////////////////////////////////////
  // Ingame action
  //////////////////////////////////////////////////////////////////////////////////////////
  
  /// save string flag for future use
  ///
  /// @param name flag unique name
  /// @value value to save
  static void save_flag(const std::string& name, const std::string& value);

  /// get string flag
  /// 
  /// @param name flag name
  /// @return flag value
  static std::string get_string_flag(const std::string& name);

    /// save double flag for future use
  ///
  /// @param name flag unique name
  /// @value value to save
  static void save_flag(const std::string& name, double);
  /// get string flag
  /// 
  /// @param name flag name
  /// @return flag value
  static double get_double_flag(const std::string& name);

  ////////////////////////////////////////////////////////////////////////////////////////
  // Object action 
  ////////////////////////////////////////////////////////////////////////////////////////

  /// Get player ship id
  static double get_player_ship_id();

  /// Return name of star system where player is
  static std::string get_player_star_system();

  static void set_target(double who, double to);
  
  static void disable_weapon(double who);
  
  static void enable_weapon(double who);
  
  /// Make ship go to location 
  ///
  /// @param who ship id
  /// @param x point coord
  /// @param y point coord
  /// @param dist stop this order when to distance to point less then dist
  /// @return event id
  static int go_to_point(double who, double x, double y, double dist);

  /// Make ship go to location 
  ///
  /// @param who ship id
  /// @param to location id to follow
  /// @param dist stop this order when to distance to point less then dist
  /// @return event id
  static int go_to_location(double who, double to, double dist);

  /// Make ship who guard location to (prety dumb)
  ///
  /// @param who ship id
  /// @param to location id
  /// @param dist distance ship allowed to go from location
  static int guard_location(double who, double to, double dist);

  /// Make ship who guard position (x,y) (prety dumb)
  ///
  /// @param who ship id
  /// @param x point coord
  /// @param y point coord
  /// @param dist distance ship allowed to go from location
  static int guard_point(double who, double x, double y, double dist);
  
  static void set_ship_team(double who, double team);

  static int get_enemy_team();
  static int get_player_team();

  /// Remove location from game
  ///
  /// @param id location id 
  static void remove_location(double id);

  /// Set new crew number to ship
  ///
  /// @param id ship id
  /// @param crew new crew
  static void set_crew(double id, double crew);

  /// Get x position of location
  ///
  /// @param id of location
  static int get_location_pos_x(double id);

  /// Get y position of location
  ///
  /// @param id of location
  static int get_location_pos_y(double id);

  /// Return true if location exist, false otherwise
  ///
  /// @param id of location
  static bool exist(double id);

  /// The game is finished, return to main menu
  static void end_game();

  /// Scroll text
  ///
  /// @param text vector of text strings
  /// @param background background picture
  static void scroll_text(const std::vector<std::string>& text, const std::string& background);

  /// Print hint message on screen
  ///
  /// @param text message
  /// @time how long to display
  static void print_message(const std::string& text, int time);

  static void set_camera_mode(const std::string& mode);
};


#endif

